# © 2022 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import api, fields, models


class ScopInstanceWizard(models.TransientModel):
    _name = "scop.instance.wizard"
    _description = "Wizard: Associer des coopératives à une instance"

    instance_id = fields.Many2one(
        comodel_name="scop.instance", string="Instance", required=True
    )
    partner_ids = fields.Many2many(
        comodel_name="res.partner",
        string="Coopératives",
        domain="[('is_company', '=', True)]",
    )
    type = fields.Selection(
        [
            ("all", 'Toutes les coops en statut "Soumis CG"'),
            ("select_5_cg", 'Sélectionner des coops en statut "Soumis CG"'),
            ("select_all", "Sélectionner parmi toutes les coops"),
        ],
        string="Type de sélection",
        default="all",
    )

    # ------------------------------------------------------
    # Onchange
    # ------------------------------------------------------
    @api.onchange("type")
    def onchange_type(self):
        self.instance_id = self.env["scop.instance"].browse(
            self.env.context.get("active_id")
        )
        already_linked_ids = self.instance_id.instance_partner_ids.mapped(
            "partner_id"
        ).ids
        if self.type == "select_5_cg":
            available_soumis_cg = self.env["res.partner"].search(
                [
                    ("is_cooperative", "=", True),
                    ("id", "not in", already_linked_ids),
                    ("membership_status", "=", "soumis_cg"),
                ]
            )
            return {
                "domain": {
                    "partner_ids": [("id", "in", available_soumis_cg.mapped("id"))]
                }
            }
        else:
            available_coops = self.env["res.partner"].search(
                [
                    ("is_cooperative", "=", True),
                    ("id", "not in", already_linked_ids),
                ]
            )
            return {
                "domain": {"partner_ids": [("id", "in", available_coops.mapped("id"))]}
            }

    # ------------------------------------------------------
    # Button function
    # ------------------------------------------------------
    def add_partners(self):
        self.ensure_one()
        self.instance_id = self.env["scop.instance"].browse(
            self.env.context.get("active_id")
        )
        if self.type == "all":
            already_linked_ids = self.instance_id.instance_partner_ids.mapped(
                "partner_id"
            ).ids
            self.partner_ids = self.env["res.partner"].search(
                [
                    ("is_cooperative", "=", True),
                    ("id", "not in", already_linked_ids),
                    ("membership_status", "=", "soumis_cg"),
                ]
            )
        # Write instance in partner
        self.partner_ids.write({"instance_ids": [(4, self.instance_id.id)]})
        # Write partner in instance
        for partner_id in self.partner_ids:
            self.env["scop.instance.partner"].create(
                {
                    "instance_id": self.instance_id.id,
                    "partner_id": partner_id.id,
                }
            )
